import { useState, useEffect } from "react";
import { useNavigate } from "react-router-dom";
import { cambioPrecioBebida, deleteBebida, getAllBebidas, ingresoBebida, retiroBebida } from "./bebidas-service";
import Swal from 'sweetalert2'

export default function AbmBebidas() {
    const [error, setError] = useState(null);
    const [bebidas, setBebidas] = useState([]);
    const navigate = useNavigate();
    const [filtro, setFiltro] = useState('');
    const [bebidasFiltradas, setBebidasFiltradas] = useState([]);

    useEffect(() => {
        const listarBebidas = async () => {
            try {
                const listaBebidas = await getAllBebidas();
                setBebidas(listaBebidas.bebidas);
                setBebidasFiltradas(listaBebidas.bebidas);
            } catch (error) {
                setError(error.message);
                console.error('error al listar bebidas: ', error.message)
            }
        }
        listarBebidas();
    }, [])

    useEffect(() => {
        const filtrarBebidas = async () => {
            const listaBebidasFiltradas = bebidas.filter((bebida) => bebida.nombre.toLowerCase().startsWith(filtro.toLowerCase()));
            if (filtro.length > 0)
                setBebidasFiltradas(listaBebidasFiltradas);
            else
                setBebidasFiltradas(bebidas);
        }
        filtrarBebidas()
    }, [filtro, bebidas])

    function EliminarBebida(bebidaID) {
        Swal.fire({
            icon: 'warning',
            title: '¿Estás seguro de que quieres eliminar la bebida?',
            showCancelButton: true,
            confirmButtonColor: "#3085d6",
            cancelButtonColor: "#d33",
            confirmButtonText: 'Si, borrar'
        }).then((result) => {
            if (result.isConfirmed) {
                try {
                    deleteBebida(bebidaID);
                    Swal.fire({
                        title: 'Bebida eliminada',
                        icon: 'success',
                        timer: 3000,
                        showConfirmButton: false
                    })
                    setTimeout(() => { window.location.reload() }, 1700)

                } catch (error) {
                    Swal.fire({
                        icon: 'error',
                        title: error.message
                    })
                }

            }
        })
    }

    async function IngresoBebida(bebidaID) {
        const { value: cantidad } = await Swal.fire({
            title: 'Ingresa cantidad',
            input: 'text',
            inputLabel: 'Cantidad que ingresa',
            showCancelButton: true,
        });
        if (cantidad) {
            const cantidadNum = parseInt(cantidad)
            if (isNaN(cantidadNum) || cantidadNum <= 0) {
                Swal.fire({
                    icon: 'error',
                    text: 'La cantidad ingresada no es válida'
                })
            }
            else {
                try {
                    await ingresoBebida(bebidaID, cantidad);
                    window.location.reload();
                } catch (error) {
                    console.error(error)
                    Swal.fire({
                        icon: 'error',
                        text: `Error al agregar la bebida:  ${error.response.data.detail}`
                    })
                }
            }
        }

    }

    async function RetiroBebida(bebidaID) {
        const { value: cantidad } = await Swal.fire({
            title: 'Ingresa cantidad',
            input: 'text',
            inputLabel: 'Cantidad que retira',
            showCancelButton: true,
        });
        if (cantidad) {
            const cantidadNum = parseInt(cantidad)
            if (isNaN(cantidadNum) || cantidadNum <= 0) {
                Swal.fire({
                    icon: 'error',
                    text: 'La cantidad ingresada no es válida'
                })
            }
            else {
                try {
                    await retiroBebida(bebidaID, cantidad);
                    window.location.reload();
                } catch (error) {
                    console.error(error)
                    Swal.fire({
                        icon: 'error',
                        text: `Error al retirar la bebida:  ${error.response.data.detail}`
                    })
                }
            }
        }

    }

    async function EditarPrecioBebida(bebidaID, precio) {
        const { value: nuevoPrecio } = await Swal.fire({
            title: 'Ingresa el nuevo precio',
            input: 'text',
            text: `El precio anterior es ${precio}`,
            showCancelButton: true,
        });
        if (nuevoPrecio) {
            const nuevoPrecioNum = parseInt(nuevoPrecio)
            if (isNaN(nuevoPrecioNum) || nuevoPrecioNum <= 0) {
                Swal.fire({
                    icon: 'error',
                    text: 'El precio ingresado no es válido'
                })
            }
            else {
                try {
                    await cambioPrecioBebida(bebidaID, nuevoPrecioNum);
                    window.location.reload();
                } catch (error) {
                    Swal.fire({
                        icon: 'error',
                        text: `Error al cambiar el precio de la bebida:  ${error.response.data.detail}`
                    })
                }
            }
        }

    }

    return (
        <div className="container text-center border border-5 rounded-5 mt-3">
            {error ? <h1 className="text-danger">{error}</h1> : null}
            <h1 className="mt-3 mb-3 text-center">Administración de Bebidas</h1>
            <div className="row mx-3">
                <div className="col-auto">
                    <label htmlFor="nombreProducto" className="col-form-label">Filtro por nombre</label>
                </div>
                <div className="col-auto">
                    <input
                        type="text"
                        id="nombreProducto"
                        placeholder="nombre del producto"
                        value={filtro}
                        onChange={(e) => setFiltro(e.target.value)}
                        className="form-control mb-3" />
                </div>
                <div className="col-auto">
                    <button className="btn btn-danger" onClick={() => setFiltro('')}>
                        <i className="fas fa-trash-alt"></i>
                    </button>
                </div>
            </div>
            <div className="table-responsive overflow-auto mx-4 mb-4" style={{ maxHeight: '300px' }}>
                <table className="table table-dark table-striped">
                    <thead className="" style={{ position: 'sticky', top: 0, backgroundColor: 'whitesmoke' }}>
                        <tr className="text-center fs-4 fst-italic text-decoration-underline">
                            <th>Producto</th>
                            <th>Precio</th>
                            <th>Cantidad</th>
                            <th></th>
                        </tr>
                    </thead>
                    <tbody>
                        {bebidasFiltradas.sort((function (a, b) {
                            if (a.nombre > b.nombre) return 1;
                            if (a.nombre < b.nombre) return -1;
                            return 0
                        })).map((bebida) => (
                            <tr key={bebida.id}>
                                <td className="text-center">{bebida.nombre}</td>
                                <td className="text-center">{bebida.precio}</td>
                                <td className="text-center">{bebida.cantidad}</td>
                                <td className="align-items-center">
                                    <div className="btn-group" role="group" aria-label="Basic example">
                                        <button className="btn btn-info mr-2" onClick={() => IngresoBebida(bebida.id)}>Ingresar</button>
                                        <button className="btn btn-info mr-2" onClick={() => RetiroBebida(bebida.id)}>Retirar</button>
                                        <button className="btn btn-secondary" onClick={() => EditarPrecioBebida(bebida.id, bebida.precio)}>Editar Precio</button>
                                        <button className="btn btn-danger" onClick={() => EliminarBebida(bebida.id)}>Eliminar</button>
                                    </div>

                                </td>
                            </tr>
                        ))}
                    </tbody>
                </table>
            </div>
            <div className="d-flex mb-3 justify-content-center align-items-center">
                <button className="btn btn-success" onClick={() => navigate('agregar-bebida')}>Agregar Bebida Nueva</button>
            </div>
        </div>
    );
}