import { useEffect, useState } from "react";
import { useNavigate } from "react-router-dom";
import { eliminarEvento, getAllEventos } from "./eventos-service";
import Swal from 'sweetalert2'
import './tablaEventos.css'
import '@fortawesome/fontawesome-free/css/all.min.css'


export default function AbmEventos() {

    const [error, setError] = useState(null);
    const [eventos, setEventos] = useState([]);
    const [eventosFiltrados, setEventosFiltrados] = useState([])
    const [filtro, setFiltro] = useState('')
    const navigate = useNavigate();

    const yesterday = new Date();
    yesterday.setDate(yesterday.getDate() - 1);
    const yesterdayTime = yesterday.getTime();

    useEffect(() => {
        const listarEvento = async () => {
            try {
                const listaEventos = await getAllEventos();
                const listaEventosFuturos = listaEventos.eventos.filter((evento) => yesterdayTime < new Date(evento.fecha).getTime())
                setEventos(listaEventosFuturos);
                setEventosFiltrados(listaEventosFuturos)
            } catch (error) {
                setError(error.message)
            }
        }
        listarEvento();
    }, [yesterdayTime])

    useEffect(() => {
        const filtrarEventos = async () => {
            try {
                const listaEventosFiltrados = eventos.filter((evento) => evento.fecha === filtro);
                if (filtro.length > 0) {
                    setEventosFiltrados(listaEventosFiltrados);
                } else {
                    setEventosFiltrados(eventos);
                }
            } catch (error) {
                setError(error);
            }
        }
        filtrarEventos();
    }, [filtro, eventos])

    function VerInfoEvento(eventoID) {
        navigate(`/eventos/info/${eventoID}`);
    }
    function EliminarEvento(eventoID) {
        Swal.fire({
            icon: 'warning',
            title: '¿Estás seguro de que quieres cancelar el evento?',
            text: 'Si preciona aceptar, las bebidas seran devueltas al deposito',
            showCancelButton: true,
            confirmButtonColor: "#3085d6",
            cancelButtonColor: "#d33",
            confirmButtonText: 'Si, cancelar'
        }).then((result) => {
            if (result.isConfirmed) {
                try {
                    eliminarEvento(eventoID);
                    Swal.fire({
                        title: 'Evento eliminado',
                        icon: 'success',
                        timer: 3000,
                        showConfirmButton: false
                    })
                    setTimeout(() => { window.location.reload() }, 1700)

                } catch (error) {
                    Swal.fire({
                        icon: 'error',
                        title: error.message
                    })
                }

            }
        })
    }


    return (
        <div className="container text-center border border-5 rounded-5 mt-3">
            {error ? <h1 className="text-danger">{error}</h1> : null}
            <h1 className="mt-3 mb-3 text-center">Eventos</h1>
            <div className="row mb-3 mx-3">
                <div className="col-auto">
                    <label htmlFor="fecha" className="col-form-label">Filtrar por fecha</label>
                </div>
                <div className="col-auto">
                    <input type="date" id="fecha" value={filtro} className="form-control" onChange={(e) => setFiltro(e.target.value)} />
                </div>
                <div className="col-auto">
                    <button className="btn btn-danger" onClick={() => setFiltro('')}>
                        <i className="fas fa-trash-alt"></i>
                    </button>
                </div>
            </div>
            <div className="table-responsive overflow-auto mx-4" style={{ maxHeight: '300px' }}>
                <table className="table table-dark table-striped">
                    <thead className="" style={{ position: 'sticky', top: 0, backgroundColor: 'whitesmoke' }}>
                        <tr className="text-center fs-4 fst-italic text-decoration-underline">
                            <th>Lugar</th>
                            <th>Fecha</th>
                            <th></th>
                        </tr>
                    </thead>
                    <tbody>
                        {eventosFiltrados
                            .sort((function (a, b) {
                                if (a.fecha > b.fecha) return 1;
                                if (a.fecha < b.fecha) return -1;
                                return 0
                            }))
                            .map((evento) => (
                                <tr key={evento.id}>
                                    <td className="text-center">{evento.lugar}</td>
                                    {Date.now() > new Date(evento.fecha).getTime() ? <td className="text-center text-danger">{evento.fecha}</td> : <td className="text-center">{evento.fecha}</td>}
                                    <td className="align-items-center ancho-columna">
                                        <div className="btn-group" role="group" aria-label="Basic example">
                                            <button className="btn btn-info mr-2" onClick={() => VerInfoEvento(evento.id)}>Info Evento</button>
                                            <button className="btn btn-danger mr-2" onClick={() => EliminarEvento(evento.id)}>Cancelar Evento</button>
                                        </div>

                                    </td>
                                </tr>
                            ))}
                    </tbody>
                </table>
            </div>
            <div className="d-flex my-3 justify-content-center align-items-center">
                <button className="btn btn-success" onClick={() => navigate('agregar-evento')}>Agregar Evento Nuevo</button>
            </div>
        </div>
    );
}