from datetime import date
from typing import List
from fastapi import FastAPI, HTTPException
from fastapi.middleware.cors import CORSMiddleware
from sqlalchemy import CheckConstraint, Column, Date, Float, ForeignKey, Integer, String, create_engine, event
from sqlalchemy.orm import sessionmaker, declarative_base, relationship, Session
from components.modelos.models import BebidaBase, InfoEventoTerminado, ListaBebidas, BebidaSinId, EventoBase, ListaBebidasXEvento, ListaEventos, EventoSinId, BebidaXEventoBase

app = FastAPI();

app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],  
    allow_credentials=True,
    allow_methods=["*"],  
    allow_headers=["*"],  
)

usuario = "postgres"
password = "123"

SQLALCHEMY_DATABASE_URL = f"postgresql+psycopg2://{usuario}:{password}@localhost:5432/postgres"
engine = create_engine(SQLALCHEMY_DATABASE_URL, echo=True)
SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)

Base = declarative_base()

class Bebida(Base):
    __tablename__ = 'bebidas'
    id = Column(Integer, primary_key=True, index=True)
    nombre = Column(String(50), index=True, unique=True)
    precio = Column(Integer)
    cantidad = Column(Integer, CheckConstraint('cantidad > 0', name='cantidad_positive'))
    eventos_asignados = relationship('BebidaXEvento', back_populates= 'bebida')
    
    @staticmethod
    def validate_cantidad_before_update(mapper, connection, target):
        if (target.cantidad < 0):
            raise ValueError('La cantidad no puede ser negativa')
            
        
    

class Evento(Base):  
    __tablename__ = 'eventos'
    id = Column(Integer, primary_key=True)
    lugar = Column(String(50))
    fecha = Column(Date)
    empleados = Column(Integer)
    horas = Column(Float)
    paga_empleados = Column(Integer)
    bebidas_asignadas = relationship('BebidaXEvento', back_populates= 'evento')

class BebidaXEvento(Base):
    __tablename__ = 'bebidaXevento'
    idEvento = Column(Integer, ForeignKey('eventos.id'), primary_key=True)
    idBebida = Column(Integer, ForeignKey('bebidas.id'), primary_key=True)
    cantidad = Column(Integer)
    bebida = relationship('Bebida', back_populates= 'eventos_asignados')
    evento = relationship('Evento', back_populates= 'bebidas_asignadas')

    @staticmethod
    def check_cantidad_before_update(mapper, connection, target):
        if target.cantidad < 0:
            raise ValueError('La cantidad no puede ser negativa')  

event.listen(BebidaXEvento, 'before_update', BebidaXEvento.check_cantidad_before_update)
event.listen(Bebida, 'before_update', Bebida.validate_cantidad_before_update)


#Base.metadata.drop_all(engine)    
Base.metadata.create_all(engine)

session = SessionLocal()

class BebidasRepository():
    def buscar_bebida(self, session, id: int) -> Bebida:
        bebidaBuscada = session.query(Bebida).filter(Bebida.id == id).first()
        if bebidaBuscada is None:
            raise HTTPException(status_code=404, detail="Bebida no encontrado")
        return bebidaBuscada
    
    
    def all_bebidas(self, session) -> List[Bebida]:
        bebidas = session.query(Bebida).all()
        return bebidas
    
    def agregar_bebida(self, session, bebida: Bebida) -> Bebida:
        try:
            session.add(bebida)
            session.commit()
            return bebida
            ##session.refresh(bebida)
        except Exception as e:
            session.rollback()
            print(f"Error: {e}")
            raise HTTPException(status_code=404, detail='error al agregar bebida')
        
    def editar_precio_bebida (self, session, id: int, precio: int) -> Bebida:
        bebidaEditada = session.query(Bebida).filter(Bebida.id == id).first()
        if bebidaEditada is None:
            raise HTTPException(status_code=404, detail="No se encontro la bebida")
        bebidaEditada.precio = precio
        session.commit()
        return bebidaEditada
    
    def agregar_cantidad_bebida (self, session, id: int, cantidadAgregada: int) -> Bebida:
        bebidaEditada = session.query(Bebida).filter(Bebida.id == id).first()
        if bebidaEditada is None:
            raise HTTPException(status_code=404, detail="No se encontro la bebida")
        
        if not isinstance(cantidadAgregada, int):
            try:
                cantidadAgregada = int(cantidadAgregada)
            except ValueError:
                raise HTTPException(status_code=400, detail=f"cantidadAgregada debe ser un entero, se recibió {type(cantidadAgregada)}")

        bebidaEditada.cantidad += cantidadAgregada
        session.commit()
        return bebidaEditada
    
    def retirar_cantidad_bebida (self, session, id: int, cantidadRetirada: int) -> Bebida:
        bebidaEditada = session.query(Bebida).filter(Bebida.id == id).first()
        if bebidaEditada is None:
            raise HTTPException(status_code=404, detail="No se encontro la bebida")
        
        if not isinstance(cantidadRetirada, int):
            try:
                cantidadRetirada = int(cantidadRetirada)
            except ValueError:
                raise HTTPException(status_code=400, detail=f"cantidadRetirada debe ser un entero, se recibió {type(cantidadRetirada)}")

        if bebidaEditada.cantidad < cantidadRetirada:
            raise HTTPException(status_code=404, detail="No hay tantas bebidas en el deposito")
        bebidaEditada.cantidad -= cantidadRetirada
        session.commit()
        return bebidaEditada
    
    def eliminar_bebida (self, session, id: int) -> Bebida:
        bebidaBorrada = session.query(Bebida).filter(Bebida.id == id).first()
        if bebidaBorrada is None:
            raise HTTPException(status_code=404, detail="No se encontro la bebida")
        
        bebidasEnEventos = session.query(BebidaXEvento).filter(BebidaXEvento.idBebida == id).all()
        for bebidaXevento in bebidasEnEventos:
            session.delete(bebidaXevento)
        
        session.delete(bebidaBorrada)
        session.commit()
        return bebidaBorrada
    
    #def retirar_cantidad_bebida (self, session, id: int, cantidadRetirada: int) -> Bebida:
    #    bebidaEditada = session.query(Bebida).filter(Bebida.id == id).first()
    #    if bebidaEditada is None:
    #        raise HTTPException(status_code=404, detail="No se encontro la bebida")
    #    if (bebidaEditada.cantidad < cantidadRetirada):
    #        raise HTTPException(status_code=404, detail="Cantidad no disponible")
    #    bebidaEditada.cantidad -= cantidadRetirada
    #    session.commit()
    #    return bebidaEditada
    
class EventosRepository():
    def all_eventos(self, session) -> List[Evento]:
        eventos = session.query(Evento).all()
        return eventos
    
    def buscar_evento(self, session, id: int) -> Evento:
        eventoBuscado = session.query(Evento).filter(Evento.id == id).first()
        if eventoBuscado is None:
            raise HTTPException(status_code=404, detail="No se encontro el evento")
        return eventoBuscado
    
    def eliminar_evento(self, session, id: int) -> Evento:
        eventoEliminado = session.query(Evento).filter(Evento.id == id).first()
        if eventoEliminado is None:
            raise HTTPException(status_code=404, detail="No se encontro el evento")
        
        eventosConBebida = session.query(BebidaXEvento).filter(BebidaXEvento.idEvento == id).all()
        for bebidaXevento in eventosConBebida:
            session.delete(bebidaXevento)
        
        session.delete(eventoEliminado)
        session.commit()
        return eventoEliminado
    
    def fin_evento(self, session, id: int) -> InfoEventoTerminado:
        eventoTerminado = session.query(Evento).filter(Evento.id == id).first()
        if eventoTerminado is None:
            raise HTTPException(status_code=404, detail="No se encontro el evento")
        
        ingresosTotales = 0   
        pagaEmpleados = 0
        ingresosBrutos = 0  
        bebidaXeventoAeliminar = session.query(BebidaXEvento).filter(BebidaXEvento.idEvento == id).all()
        
        if bebidaXeventoAeliminar:
            for bebidaXevento in bebidaXeventoAeliminar:
                bebida = session.query(Bebida).filter(Bebida.id == bebidaXevento.idBebida).first()
                if bebida:
                    ingresosTotales += bebida.precio*bebidaXevento.cantidad
                #session.delete(bebidaXevento)
            
        pagaEmpleados = eventoTerminado.horas * eventoTerminado.paga_empleados * eventoTerminado.empleados
        ingresosBrutos = ingresosTotales - pagaEmpleados
        
        #session.delete(eventoTerminado)
        session.commit()
        return InfoEventoTerminado(ingresos_totales=ingresosTotales, paga_empleados=pagaEmpleados, ingresos_bruto=ingresosBrutos)
    
    def agregar_evento(self, session, eventoNuevo: Evento) -> Evento:
        try:
            session.add(eventoNuevo)
            session.commit()
            return eventoNuevo
        except Exception as e:
            print(f"Error: {e}")
            session.rollback()
            raise HTTPException(status_code=404, detail="No se pudo agregar el evento")
        
class BebidaXEventoRepository():
    def all_bebidas_evento(self, session, idEvento: int) -> List[BebidaXEventoBase]:
        listaBebidasEvento = session.query(BebidaXEvento).filter(BebidaXEvento.idEvento == idEvento).all()
        bebidasXevento = []
        
        for bXe in listaBebidasEvento: 
            bebida= session.query(Bebida).filter(Bebida.id == bXe.idBebida).first()
            if bebida:
                bebidaXeventoInfo = BebidaXEventoBase(idBebida=bebida.id, nombreBebida=bebida.nombre, cantidad=bXe.cantidad)
                bebidasXevento.append(bebidaXeventoInfo)

        return bebidasXevento

    
    def agregar_bebida_evento(self, session, idEvento: int, idBebida: int, cantidad: int) -> BebidaXEventoBase:
        bebida = session.query(Bebida).filter(Bebida.id == idBebida).first()
        bebidaXevento_existente = session.query(BebidaXEvento).filter(BebidaXEvento.idEvento == idEvento, BebidaXEvento.idBebida == idBebida).first()
        if bebida is None:
            raise HTTPException(status_code=404, detail="No se encuentra la bebida")       

        if bebida.cantidad < cantidad:
            raise HTTPException(status_code=404, detail="Cantidad solicitada mayor a la cantidad disponible")
        
        if bebidaXevento_existente:
            bebida.cantidad -= cantidad
            bebidaXevento_existente.cantidad += cantidad
            session.commit()
            return BebidaXEventoBase(idBebida= bebida.id, nombreBebida= bebida.nombre, cantidad= bebidaXevento_existente.cantidad)
        else:
            bebida.cantidad -= cantidad
            bebidaXevento_nuevo = BebidaXEvento(idEvento = idEvento, idBebida = idBebida, cantidad = cantidad)
            session.add(bebidaXevento_nuevo)
            session.commit()
            return BebidaXEventoBase(idBebida= bebida.id, nombreBebida= bebida.nombre, cantidad= bebidaXevento_nuevo.cantidad)
            
    def retirar_bebida_evento (self, session, idEvento: int, idBebida: int, cantidad: int) -> BebidaXEventoBase:
        bebidaXevento = session.query(BebidaXEvento).filter(BebidaXEvento.idEvento == idEvento, BebidaXEvento.idBebida == idBebida).first()
        bebida = session.query(Bebida).filter(Bebida.id == idBebida).first()
        if bebida is None:
            raise HTTPException(status_code=404, detail="No se encontro la bebida")
        if bebidaXevento is None:
            raise HTTPException(status_code=404, detail="No se encontro la bebida en el evento")  
        if bebidaXevento.cantidad < cantidad:
            raise HTTPException(status_code=404, detail="Cantidad retirada mayor a cantidad disponible")
        bebidaXevento.cantidad -= cantidad
        bebida.cantidad += cantidad
        session.commit()
        return BebidaXEventoBase(idBebida= bebida.id, nombreBebida=bebida.nombre, cantidad= bebidaXevento.cantidad)
    
    
####Bebidas####

@app.get ('/bebidas', response_model = ListaBebidas)
async def get_all_bebidas():
    repo = BebidasRepository()
    bebidas = repo.all_bebidas(session)
    return ListaBebidas(bebidas= [BebidaBase(id = bebida.id, nombre=bebida.nombre, precio=bebida.precio, cantidad=bebida.cantidad) for bebida in bebidas])

@app.get ('/bebidas/{idBebida}', response_model = BebidaBase)
async def get_bebida(idBebida: int):
    repo = BebidasRepository()
    try:
        bebida = repo.buscar_bebida(session, idBebida)
        return bebida
    except HTTPException as e:
        raise e  

@app.post ('/bebidas', response_model = BebidaBase)
async def post_bebida(bebida: BebidaSinId):
    repo = BebidasRepository()
    try:
        ##bebida = BebidaSinId(nombre=nombre, precio=precio, cantidad=cantidad)
        bebida_nueva = Bebida(nombre = bebida.nombre, precio = bebida.precio, cantidad = bebida.cantidad)
        bebida_guardada = repo.agregar_bebida(session, bebida_nueva)
        return bebida_guardada
    except Exception as e:
        raise e

    
    
@app.put ('/bebidas/precio/{idBebida}', response_model = BebidaBase)
async def put_precio_bebida(idBebida: int, nuevo_precio = int):
    repo = BebidasRepository()
    try:
        bebida_editada = repo.editar_precio_bebida(session, id = idBebida, precio = nuevo_precio)
        return bebida_editada
    except HTTPException as e:
        raise e  
    
@app.put ('/bebidas/agregar_cantidad/{idBebida}', response_model = BebidaBase)
async def put_agregar_cantidad_bebida(idBebida: int, cantidad_agregada = int):
    repo = BebidasRepository()
    try:
        bebida_cant_agregada = repo.agregar_cantidad_bebida(session, id = idBebida, cantidadAgregada = cantidad_agregada)
        return bebida_cant_agregada
    except HTTPException as e:
        raise e
    except Exception as e:
        raise HTTPException(status_code=404, detail=str(e))
    
@app.put ('/bebidas/retirar_cantidad/{idBebida}', response_model = BebidaBase)
async def put_retirar_cantidad_bebida(idBebida: int, cantidad_retirada = int):
    repo = BebidasRepository()
    try:
        bebida_cant_agregada = repo.retirar_cantidad_bebida(session, idBebida, cantidad_retirada)
        return bebida_cant_agregada
    except HTTPException as e:
        raise e
    except Exception as e:
        raise HTTPException(status_code=404, detail=str(e))

@app.delete ('/bebidas/{idBebida}', response_model = BebidaBase)
async def delete_bebida(idBebida: int):
    repo = BebidasRepository()
    bebidaBorrada = repo.eliminar_bebida(session, idBebida)
    return bebidaBorrada

#@app.put ('/bebidas/retirar_cantidad/{idBebida}', response_model = BebidaBase)
#async def put_retirar_cantidad_bebida(idBebida: int, cantidad_retirada = int):
#    repo = BebidasRepository()
#    try:
#        bebida_cant_retirada = repo.retirar_cantidad_bebida(session, id = idBebida, cantidadRetirada = cantidad_retirada)
#        return bebida_cant_retirada
#    except HTTPException as e:
#        raise e  



####Eventos####

@app.get ('/eventos', response_model = ListaEventos)
async def get_all_eventos():
    repo = EventosRepository()
    eventos = repo.all_eventos(session)
    return ListaEventos(eventos = [EventoBase(id = evento.id, lugar = evento.lugar, fecha = evento.fecha, empleados = evento.empleados, horas = evento.horas, paga_empleados = evento.paga_empleados) for evento in eventos])

@app.get ('/eventos/{idEvento}', response_model = EventoBase)
async def get_all_eventos(idEvento: int):
    repo = EventosRepository()
    try:
        evento = repo.buscar_evento(session, idEvento)
        return evento
    except HTTPException as e:
        raise e  
    except:
        raise HTTPException(status_code=102, detail='hola')
    
    
@app.post ('/eventos', response_model = EventoBase)
async def post_evento(evento: EventoSinId):
    repo = EventosRepository()
    try:
        evento_nuevo = Evento(lugar = evento.lugar, fecha = evento.fecha, empleados = evento.empleados, horas = evento.horas, paga_empleados = evento.paga_empleados)
        evento_agregado = repo.agregar_evento(session, eventoNuevo=evento_nuevo)
        return evento_agregado
    except HTTPException as e:
        raise e
    except Exception as e:
        raise HTTPException(status_code=404, detail=str(e))
    
@app.delete ('/eventos/{idEvento}', response_model = EventoBase)
async def delete_evento(idEvento: int):
    repo = EventosRepository()
    try:
        evento_eliminado = repo.eliminar_evento(session, idEvento)
        return evento_eliminado
    except HTTPException as e:
        raise e 
    except Exception as e:
        raise HTTPException(status_code=404, detail=str(e))
    
@app.get ('/eventos/fin_evento/{idEvento}', response_model= InfoEventoTerminado)  
async def terminar_evento(idEvento: int):
    repo = EventosRepository()
    try:
        infoEvento = repo.fin_evento(session, idEvento)
        return infoEvento
    except HTTPException as e:
        raise e 
    except Exception as e:
        raise HTTPException(status_code=404, detail=str(e))
    

####bebidasXeventos####

@app.get ('/bebidasXevento/{idEvento}', response_model = ListaBebidasXEvento)
async def get_bebidas_evento(idEvento: int):
    repo = BebidaXEventoRepository()
    try:
        bebidas_evento = repo.all_bebidas_evento(session, idEvento)
        return ListaBebidasXEvento(bebidasXevento= bebidas_evento)
    except HTTPException as e:
        raise e
    #except Exception as e:
    #    raise HTTPException(status_code=404, detail=str(e))
    
@app.post ('/bebidasXevento/{idEvento}', response_model=BebidaXEventoBase)
async def post_bebida_evento(idEvento: int, idBebida: int, cantidadAgregada: int):
    repo = BebidaXEventoRepository()
    try:
        return repo.agregar_bebida_evento(session, idEvento, idBebida, cantidadAgregada)
    except HTTPException as e:
        raise e
    except Exception as e:
        raise HTTPException(status_code=404, detail=str(e))
    
@app.put ('/bebidasXevento/{idEvento}', response_model= BebidaXEventoBase)
async def put_bebida_evento(idEvento: int, idBebida: int, cantidadRetirada: int):
    repo = BebidaXEventoRepository()
    try:
        return repo.retirar_bebida_evento(session, idEvento, idBebida, cantidadRetirada)
    except HTTPException as e:
        raise e
    except Exception as e:
        raise HTTPException(status_code=404, detail=str(e))
    
@app.delete ('/bebidasXevento/{idEvento}')
async def delete_bebida_evento(idEvento: int, idBebida: int):
    repo = BebidaXEventoRepository()


##validar cuando sale una bebida del deposito o cuando sale de babidasXevento para evitar usar los eventos de las tabla porque generan errores
